const isSigner = value => 'publicKey' in value;
const createSignerFromKeypair = (context, keypair) => ({
  publicKey: keypair.publicKey,
  async signMessage(message) {
    return context.eddsa.sign(message, keypair);
  },
  async signTransaction(transaction) {
    transaction.sign([keypair]);
    return transaction;
  },
  async signAllTransactions(transactions) {
    return Promise.all(transactions.map(transaction => this.signTransaction(transaction)));
  }
});
const createNoopSigner = publicKey => ({
  publicKey,
  async signMessage(message) {
    return message;
  },
  async signTransaction(transaction) {
    return transaction;
  },
  async signAllTransactions(transactions) {
    return transactions;
  }
});
const createNullSigner = () => new NullSigner();
class NullSigner {
  // TODO(loris): Custom errors.
  get publicKey() {
    throw new Error('Method not implemented.');
  }
  signMessage() {
    throw new Error('Method not implemented.');
  }
  signTransaction() {
    throw new Error('Method not implemented.');
  }
  signAllTransactions() {
    throw new Error('Method not implemented.');
  }
}

export { NullSigner, createNoopSigner, createNullSigner, createSignerFromKeypair, isSigner };
//# sourceMappingURL=Signer.mjs.map
