import type { PublicKeyInput } from './EddsaInterface';
import { DataEnum, ScalarEnum } from './Enums';
import type { PublicKey } from './PublicKey';
import type { Serializer, WrapInSerializer } from './Serializer';
import type { Nullable, Option } from './Option';
export type StructToSerializerTuple<T extends object> = Array<{
    [K in keyof T]: [K, Serializer<T[K]>];
}[keyof T]>;
export type DataEnumToSerializerTuple<T extends DataEnum> = Array<T extends any ? [
    T['__kind'],
    keyof Omit<T, '__kind'> extends never ? Serializer<Omit<T, '__kind'>> | Serializer<void> : Serializer<Omit<T, '__kind'>>
] : never>;
export interface SerializerInterface {
    tuple: <T extends any[]>(items: WrapInSerializer<[...T]>, description?: string) => Serializer<T>;
    vec: <T>(item: Serializer<T>, description?: string) => Serializer<T[]>;
    array: <T>(item: Serializer<T>, size: number, description?: string) => Serializer<T[]>;
    map: <K, V>(key: Serializer<K>, value: Serializer<V>, description?: string) => Serializer<Map<K, V>>;
    set: <T>(item: Serializer<T>, description?: string) => Serializer<Set<T>>;
    option: <T>(item: Serializer<T>, description?: string) => Serializer<Option<T>>;
    nullable: <T>(item: Serializer<T>, description?: string) => Serializer<Nullable<T>>;
    struct: <T extends object>(fields: StructToSerializerTuple<T>, description?: string) => Serializer<T>;
    enum<T>(constructor: ScalarEnum<T>, description?: string): Serializer<T>;
    dataEnum<T extends DataEnum>(fields: DataEnumToSerializerTuple<T>, description?: string): Serializer<T>;
    unit: Serializer<void>;
    bool: Serializer<boolean>;
    u8: Serializer<number>;
    u16: Serializer<number>;
    u32: Serializer<number>;
    u64: Serializer<number | bigint, bigint>;
    u128: Serializer<number | bigint, bigint>;
    i8: Serializer<number>;
    i16: Serializer<number>;
    i32: Serializer<number>;
    i64: Serializer<number | bigint, bigint>;
    i128: Serializer<number | bigint, bigint>;
    f32: Serializer<number>;
    f64: Serializer<number>;
    string: Serializer<string>;
    bytes: Serializer<Uint8Array>;
    publicKey: Serializer<PublicKey | PublicKeyInput, PublicKey>;
}
export declare class NullSerializer implements SerializerInterface {
    private readonly error;
    tuple<T extends any[]>(): Serializer<T>;
    vec<T>(): Serializer<T[]>;
    array<T>(): Serializer<T[]>;
    map<K, V>(): Serializer<Map<K, V>>;
    set<T>(): Serializer<Set<T>>;
    option<T>(): Serializer<Option<T>>;
    nullable<T>(): Serializer<Nullable<T>>;
    struct<T extends object>(): Serializer<T>;
    enum<T>(): Serializer<T>;
    dataEnum<T extends DataEnum>(): Serializer<T>;
    get unit(): Serializer<void>;
    get bool(): Serializer<boolean>;
    get u8(): Serializer<number>;
    get u16(): Serializer<number>;
    get u32(): Serializer<number>;
    get u64(): Serializer<number | bigint, bigint>;
    get u128(): Serializer<number | bigint, bigint>;
    get i8(): Serializer<number>;
    get i16(): Serializer<number>;
    get i32(): Serializer<number>;
    get i64(): Serializer<number | bigint, bigint>;
    get i128(): Serializer<number | bigint, bigint>;
    get f32(): Serializer<number>;
    get f64(): Serializer<number>;
    get string(): Serializer<string>;
    get bytes(): Serializer<Uint8Array>;
    get publicKey(): Serializer<PublicKey | PublicKeyInput, PublicKey>;
}
